@@ -155,6 +155,20 @@ int raw1394_get_nodecount(raw1394handle_t handle)
 		return ieee1394_get_nodecount(handle->mode.ieee1394);
 }
 
+int raw1394_get_speed(raw1394handle_t handle, nodeid_t node)
+{
+	if (!handle) {
+		errno = EINVAL;
+		return -1;
+	}
+	if (handle->is_fw)
+		return fw_get_speed(handle->mode.fw, node);
+	else {
+		errno = ENOSYS;
+		return -1;
+	}
+}
+
 int raw1394_get_port_info(raw1394handle_t handle, struct raw1394_portinfo *pinf,
                           int maxports)
 {
@@ -292,7 +306,7 @@ int raw1394_phy_packet_write (raw1394handle_t handle, quadlet_t data)
 		return -1;
 	}
 	if (handle->is_fw)
-		return fw_phy_packet_write(handle->mode.fw, data);
+		return fw_phy_packet_write(handle, data);
 	else
 		return ieee1394_phy_packet_write(handle, data);
 }
@@ -519,6 +533,37 @@ int raw1394_update_config_rom(raw1394handle_t handle, const quadlet_t
 			new_rom, size, rom_version);
 }
 
+int raw1394_add_config_rom_descriptor(raw1394handle_t handle, u_int32_t *token,
+		quadlet_t immediate_key, quadlet_t key,
+		const quadlet_t *data, size_t size)
+{
+	if (!handle) {
+		errno = EINVAL;
+		return -1;
+	}
+	if (handle->is_fw)
+		return fw_add_config_rom_descriptor(handle->mode.fw, token,
+			immediate_key, key, data, size);
+	else {
+		errno = ENOSYS;
+		return -1;
+	}
+}
+
+int raw1394_remove_config_rom_descriptor(raw1394handle_t handle, u_int32_t token)
+{
+	if (!handle) {
+		errno = EINVAL;
+		return -1;
+	}
+	if (handle->is_fw)
+		return fw_remove_config_rom_descriptor(handle->mode.fw, token);
+	else {
+		errno = ENOSYS;
+		return -1;
+	}
+}
+
 int raw1394_get_config_rom(raw1394handle_t handle, quadlet_t *buffer,
         size_t buffersize, size_t *rom_size, unsigned char *rom_version)
 {
@@ -760,3 +805,19 @@ int raw1394_read_cycle_timer(raw1394handle_t handle,
 		return ieee1394_read_cycle_timer(handle->mode.ieee1394,
 			cycle_timer, local_time);
 }
+
+int raw1394_read_cycle_timer_and_clock(raw1394handle_t handle,
+	u_int32_t *cycle_timer, u_int64_t *local_time, clockid_t clk_id)
+{
+	if (!handle) {
+		errno = EINVAL;
+		return -1;
+	}
+	if (handle->is_fw)
+		return fw_read_cycle_timer_and_clock(handle->mode.fw,
+			cycle_timer, local_time, clk_id);
+	else {
+		errno = ENOSYS;
+		return -1;
+	}
+}
