@@ -14,7 +14,9 @@
 
 #include <stdlib.h>
 #include <byteswap.h>
-#include <linux/firewire-cdev.h>
+
+#include "firewire-constants.h"
+#include "firewire-cdev.h"
 #include "raw1394.h"
 #include "csr.h"
 #include "config.h"
@@ -69,7 +71,6 @@ struct device {
 
 struct request_closure {
 	void *data;
-	size_t length;
 	unsigned long tag;
 };
 
@@ -79,6 +80,7 @@ struct fw_handle {
 	struct port ports[MAX_PORTS];
 	int port_count;
 	int err;
+	int card;
 	int generation;
 	int abi_version;
 	void *userdata;
@@ -109,6 +111,7 @@ struct fw_handle {
 		char *filename;
 		struct epoll_closure closure;
 		int fd;
+		unsigned kernel_handle;
 		int type;
 		int irq_interval;
 		int packet_phase;
@@ -141,6 +144,7 @@ int fw_get_fd(fw_handle_t handle);
 nodeid_t fw_get_local_id(fw_handle_t handle);
 nodeid_t fw_get_irm_id(fw_handle_t handle);
 int fw_get_nodecount(fw_handle_t handle);
+int fw_get_speed(fw_handle_t handle, nodeid_t node);
 int fw_get_port_info(fw_handle_t handle, struct raw1394_portinfo *pinf,
 		   int maxports);
 int fw_set_port(fw_handle_t handle, int port);
@@ -157,9 +161,6 @@ int fw_arm_get_buf(fw_handle_t handle, nodeaddr_t start,
 		   size_t length, void *buf);
 int fw_echo_request(fw_handle_t handle, quadlet_t data);
 int fw_wake_up(fw_handle_t handle);
-int fw_phy_packet_write (fw_handle_t handle, quadlet_t data);
-int fw_start_phy_packet_write(fw_handle_t handle,
-		   quadlet_t data, unsigned long tag);
 int fw_start_read(fw_handle_t handle, nodeid_t node, nodeaddr_t addr,
 		   size_t length, quadlet_t *buffer, unsigned long tag);
 int fw_start_write(fw_handle_t handle, nodeid_t node, nodeaddr_t addr,
@@ -174,6 +175,8 @@ int fw_start_async_stream(fw_handle_t handle, unsigned int channel,
 		   unsigned int tag, unsigned int sy,
 		   unsigned int speed, size_t length, quadlet_t *data,
 		   unsigned long rawtag);
+int fw_start_phy_packet_write(fw_handle_t handle,
+		   quadlet_t data, unsigned long tag);
 int fw_start_async_send(fw_handle_t handle,
 		   size_t length, size_t header_length,
 		   unsigned int expect_response,
@@ -191,6 +194,7 @@ int fw_lock64(raw1394handle_t handle, nodeid_t node, nodeaddr_t addr,
 int fw_async_stream(raw1394handle_t handle, unsigned int channel,
 		   unsigned int tag, unsigned int sy, unsigned int speed,
 		   size_t length, quadlet_t *data);
+int fw_phy_packet_write(raw1394handle_t handle, quadlet_t data);
 int fw_async_send(fw_handle_t handle,
 		   size_t length, size_t header_length,
 		   unsigned int expect_response,
@@ -199,6 +203,10 @@ int fw_start_fcp_listen(fw_handle_t handle);
 int fw_stop_fcp_listen(fw_handle_t handle);
 int fw_update_config_rom(fw_handle_t handle, const quadlet_t *new_rom,
 		   size_t size, unsigned char rom_version);
+int fw_add_config_rom_descriptor(fw_handle_t handle, u_int32_t *token,
+		   quadlet_t immediate_key, quadlet_t key,
+		   const quadlet_t *data, size_t size);
+int fw_remove_config_rom_descriptor(fw_handle_t handle, u_int32_t token);
 int fw_get_config_rom(fw_handle_t handle, quadlet_t *buffer,
 		   size_t buffersize, size_t *rom_size,
 		   unsigned char *rom_version);
@@ -244,7 +252,9 @@ int fw_iso_recv_unlisten_channel(fw_handle_t handle,
 int fw_iso_recv_set_channel_mask(fw_handle_t handle, u_int64_t mask);
 void fw_iso_stop(fw_handle_t handle);
 void fw_iso_shutdown(fw_handle_t handle);
-int fw_read_cycle_timer(fw_handle_t handle,
-			u_int32_t *cycle_timer, u_int64_t *local_time);
+int fw_read_cycle_timer(fw_handle_t handle, u_int32_t *cycle_timer,
+			  u_int64_t *local_time);
+int fw_read_cycle_timer_and_clock(fw_handle_t handle, u_int32_t *cycle_timer,
+			  u_int64_t *local_time, clockid_t clk_id);
 
 #endif
